/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/
/*----------------------------------------------------------------------
Portions from cpp_example.cpp, part of the MusicBrainz Client Library 2.0.1
  on 05/02/03
  Original Copyright Notice:

   MusicBrainz -- The Internet music metadatabase

   Copyright (C) 2000 Robert Kaye

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

     $Id: qcdlookup.cpp,v 1.14 2004/06/17 18:03:22 echeslack Exp $

----------------------------------------------------------------------------*/
#include "qcdlookup.h"
#include <qapplication.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <musicbrainz/musicbrainz.h>

QCDLookup::QCDLookup(QWidget* parent, const char* name)
	: QVBox(parent, name)
{
	//set the title for this window
	this->setCaption("QMBTagger - CD LookUp");

	//add labels to widget
	numCD = new QLabel("CDs Found:", this, "NumCD");
	albumName = new QLabel("Album Name:", this, "AlbumName");
	albumID = new QLabel("Album ID:", this, "AlbumID");

	//contentlist: private member of QCDLookup, holds info about cd
	contentlist = new QListView(this, "ContentList");
	contentlist->addColumn("Artist");
	contentlist->addColumn("Track #");
	contentlist->addColumn("Track Title");
	contentlist->addColumn("Track ID");

	//button for initiating cdlookup, connected to public slot QCDLookup::CDLookup
	lookupbutton = new QPushButton("Look Up CD Contents", this, "LookupCDButton");
	connect(lookupbutton, SIGNAL(clicked()), SLOT(CDLookup()));
}

void QCDLookup::CDLookup()
{
	MusicBrainz mb;
	QString error;
	std::string data;	//some funkiness here, can we convert to using only QStrings?
	bool ret;
	int numTracks;

	//clear the list out
	contentlist->clear();
	qApp->processEvents();

	//change server if environment var is set, otherwise defaults to mm.musicbrainz.org:80
	if (getenv("MB_SERVER"))
	{
		QString server(getenv("MB_SERVER"));
		mb.SetServer(server.latin1(), 80);
	}

	//turn of UTF8, use ISO8859-1 instead
	mb.UseUTF8(false);

	//Query server for CD information
	ret = mb.Query(MBQ_GetCDInfo);

	//Report if query failed
	if (!ret)
	{
		string strerror;
		mb.GetQueryError(strerror);
		error = QString(strerror);
		QMessageBox::information( this, "QCDLookup", error.prepend("Query failed: "));
		qWarning("QCDLookup::CDLookup(): Query Error: %s", error.latin1());
		return;
	}

	//Report if no album matches were found, otherwise report how many were found
	if (mb.DataInt(MBE_GetNumAlbums) < 1)
	{
		QMessageBox::information( this, QString("QCDLookup"), QString("This CD was not found."));
		qWarning("QCDLookup::CDLookup(): CD Not Found.");
		return;
	}
	else
	{
		numCD->setText(QString("CDs Found: ") + QString::number(mb.DataInt(MBE_GetNumAlbums)));
	}

	//Select first found album
	mb.Select(MBS_SelectAlbum, 1);

	//Get number of tracks
	numTracks = mb.DataInt(MBE_AlbumGetNumTracks);

	//set label data for Album
	albumName->setText(QString("Album Name: ") + QString(mb.Data(MBE_AlbumGetAlbumName).c_str()));
	mb.GetIDFromURL(mb.Data(MBE_AlbumGetAlbumId), data);
	albumID->setText(QString("Album ID: ") + QString(data.c_str()));


	for(int i = 1; i <= numTracks; i++)
	{
        // create a new ListViewItem in contentlist to report Artist, TrackNum, Track Name, and Track ID
		mb.GetIDFromURL(mb.Data(MBE_AlbumGetTrackId, i), data);
		QString trackNum = QString::number(mb.DataInt(MBE_AlbumGetTrackNum, i));
		new QListViewItem(contentlist, QString(mb.Data(MBE_AlbumGetArtistName, i).c_str()), trackNum, QString(mb.Data(MBE_AlbumGetTrackName, i).c_str()), QString(data.c_str()));
	}

	return;
}
