/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include "qmbconfig.h"
#include <qlayout.h>
#include <qframe.h>
#include <qvbox.h>
#include <qgrid.h>
#include <qtabwidget.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qfile.h>
#include <qstring.h>
#include <qcheckbox.h>

QMBConfig::QMBConfig(QWidget* parent, const char* name, WFlags f)
	:QVBox(parent, name, f), settings(), serverurl("musicbrainz.org"), serverport(80), proxyurl(""), proxyport(8080), filenameformat("{ARTIST} - {TRACK}.{EXT}"), newdir(""), copyacrossfilesystems(true), username(""), password("")
{
	//the tab box
	QTabWidget* tabs = new QTabWidget(this, "tabs");

	/*******START SERVER TAB*******/
	//the grid to hold the server selection
	QFrame* servergrid = new QFrame(tabs);
	QGridLayout* servergridlayout = new QGridLayout(servergrid, 7, 4);

	//server selection grid children
	QLabel* serverurllabel = new QLabel("Server URL:", servergrid);
	servergridlayout->addWidget(serverurllabel, 0, 0);
	QLabel* serverportlabel = new QLabel("Server Port:", servergrid);
	servergridlayout->addWidget(serverportlabel, 0, 2);
	/*QLineEdit* */ this->textserverurl = new QLineEdit(serverurl, servergrid);
	servergridlayout->addWidget(textserverurl, 1, 0);
	/*QSpinBox* */ this->spinserverport = new QSpinBox(0, 30000, 1, servergrid);
	spinserverport->setValue(80);
	servergridlayout->addWidget(spinserverport, 1, 2);
	QLabel* proxyurllabel = new QLabel("Proxy URL:", servergrid);
	servergridlayout->addWidget(proxyurllabel, 2, 0);
	QLabel* proxyportlabel = new QLabel("Proxy Port:", servergrid);
	servergridlayout->addWidget(proxyportlabel, 2, 2);
	/*QLineEdit* */ this->textproxyurl = new QLineEdit(proxyurl, servergrid);
	servergridlayout->addWidget(textproxyurl, 3, 0);
	/*QSpinBox* */ this->spinproxyport = new QSpinBox(0, 30000, 1, servergrid);
	spinproxyport->setValue(8080);
	servergridlayout->addWidget(spinproxyport, 3, 2);
	QLabel* usernamelabel = new QLabel("Username:", servergrid);
	servergridlayout->addWidget(usernamelabel, 4, 0);
	QLabel* passwordlabel = new QLabel("Password:", servergrid);
	servergridlayout->addWidget(passwordlabel, 4, 2);
	/*QLineEdit* */ this->textusername = new QLineEdit(username, servergrid);
	servergridlayout->addWidget(textusername, 5, 0);
	/*QLineEdit* */ this->textpassword = new QLineEdit(password, servergrid);
	this->textpassword->setEchoMode(QLineEdit::Password);
	servergridlayout->addWidget(textpassword, 5, 2);
	servergridlayout->addColSpacing(1, 10);
	QSpacerItem* servergridspacerh = new QSpacerItem(0, 0);
	servergridlayout->addMultiCell(servergridspacerh, 6, 6, 0, 2);
	QSpacerItem* servergridspacerv = new QSpacerItem(0, 0);
	servergridlayout->addMultiCell(servergridspacerv, 0, 5, 3, 3);
	servergridlayout->setRowStretch(6, 1);
	servergridlayout->setColStretch(3, 1);
	servergridlayout->addColSpacing(0, 250);
	servergridlayout->addColSpacing(2, 150);
	//and add the server tab
	tabs->addTab(servergrid, "Server");
	/*******END SERVER TAB*******/

	/*******START NAMING TAB*******/
	//the grid to hold the naming selection
	QFrame* naminggrid = new QFrame(tabs);
	QGridLayout* naminggridlayout = new QGridLayout(naminggrid, 7, 4);

	//server selection grid children
	QLabel* namingfilenamelabel = new QLabel("File Format Naming Specification:", naminggrid);
	naminggridlayout->addWidget(namingfilenamelabel, 0, 0);
	/*QLineEdit* */ this->textfilenameformat = new QLineEdit(filenameformat, naminggrid);
	naminggridlayout->addWidget(textfilenameformat, 1, 0);
	QLabel* namingconventionlabel = new QLabel("Available replacement values:\n{ARTIST} - Artist Name\n{ALBUM} - Album Name\n{TRACK} - Track Name\n{TRACKNUM} - Track Number\n{PTN} - Padded Track Number\n{EXT} - File Extension", naminggrid);
	naminggridlayout->addWidget(namingconventionlabel, 2, 0);
	QLabel* namingnewdirlabel = new QLabel("Output Directory:", naminggrid);
	naminggridlayout->addWidget(namingnewdirlabel, 3, 0);
	/*QLineEdit* */ this->textnewdir = new QLineEdit(newdir, naminggrid);
	naminggridlayout->addWidget(textnewdir, 4, 0);
	QPushButton* namingnewdirselect = new QPushButton("Select...", naminggrid);
	naminggridlayout->addWidget(namingnewdirselect, 4, 2);
	/*QCheckBox* */ this->checkcrossfilesystem = new QCheckBox("Copy when renaming mp3 across filesystems", naminggrid);
	this->checkcrossfilesystem->setChecked(true);
	naminggridlayout->addWidget(checkcrossfilesystem, 5, 0);
	naminggridlayout->addColSpacing(1, 10);
	QSpacerItem* naminggridspacerh = new QSpacerItem(0, 0);
	naminggridlayout->addMultiCell(naminggridspacerh, 6, 6, 0, 2);
	QSpacerItem* naminggridspacerv = new QSpacerItem(0, 0);
	naminggridlayout->addMultiCell(naminggridspacerv, 0, 5, 3, 3);
	naminggridlayout->setRowStretch(6, 1);
	naminggridlayout->setColStretch(0, 1);
	//naminggridlayout->setColStretch(3, 1);
	servergridlayout->addColSpacing(0, 350);
	servergridlayout->addColSpacing(2, 50);
	//and add the server tab
	tabs->addTab(naminggrid, "Naming");
	//connect the new dir select button
	connect(namingnewdirselect, SIGNAL(clicked()), this, SLOT(SelectOutputDir()));
	/*******END NAMING TAB*******/

	/*******START BUTTON BOX*******/
	//the hboxlayout to hold the apply and cancel button
	QFrame* hbuttonbox = new QFrame(this, "hbuttonbox");
	QHBoxLayout* hbuttonboxlayout = new QHBoxLayout(hbuttonbox);

	//add the buttons
	hbuttonboxlayout->addStretch(1);
	QPushButton* loaddefaults = new QPushButton("Load Defaults", hbuttonbox);
	hbuttonboxlayout->addWidget(loaddefaults);
	hbuttonboxlayout->addSpacing(10);
	QPushButton* savedefaults = new QPushButton("Save Defaults and Apply", hbuttonbox);
	hbuttonboxlayout->addWidget(savedefaults);
	hbuttonboxlayout->addSpacing(10);
	QPushButton* apply = new QPushButton("Apply", hbuttonbox);
	hbuttonboxlayout->addWidget(apply);
	hbuttonboxlayout->addSpacing(10);
	QPushButton* cancel = new QPushButton("Cancel", hbuttonbox);
	hbuttonboxlayout->addWidget(cancel);
	//connect the buttons to the slots
	connect(loaddefaults, SIGNAL(clicked()), this, SLOT(LoadDefaults()));
	connect(savedefaults, SIGNAL(clicked()), this, SLOT(SaveDefaults()));
	connect(apply, SIGNAL(clicked()), this, SLOT(Applied()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(Cancelled()));
	/*******END BUTTON BOX*******/

	//now deal with configuration
	this->LoadDefaults();
}

QMBConfig::~QMBConfig() {

}

QSettings* QMBConfig::GetSettings() {
	return &(this->settings);
}

void QMBConfig::EmitInformation() {
	//this function just emits info, useful if we need to get at the information now instead of waiting for the user to apply
	emit InformationSet(serverurl, serverport, proxyurl, proxyport, filenameformat, newdir, copyacrossfilesystems, username, password);
}

void QMBConfig::LoadDefaults() {
	this->settings.setPath("qmbtagger.sf.net", "QMBTagger");

	//prepend QMBTagger to all keys, results in $HOME/.qt/qmbtaggerrc file
	this->settings.beginGroup( "/QMBTagger");

		this->serverurl = this->settings.readEntry( "/server/url", "musicbrainz.org");
		this->serverport = this->settings.readNumEntry( "/server/port", 80);
		this->proxyurl = this->settings.readEntry( "/proxy/url", "");
		this->proxyport = this->settings.readNumEntry( "/proxy/port", 8080);
		this->filenameformat = this->settings.readEntry( "/output/format", "{ARTIST} - {TRACK}.{EXT}");
		this->newdir = this->settings.readEntry( "/output/directory", "");
		this->copyacrossfilesystems = this->settings.readBoolEntry( "/output/crossfs", true);
		this->username = this->settings.readEntry( "/id/username", "");
		this->password = this->settings.readEntry( "/id/password", "");

		//emit the signal to load the TRMS
		emit LoadTRMs();

	//end QMBTagger group
	this->settings.endGroup();

	//call RestoreOptions() to get the widgets displaying the loaded options
	this->RestoreOptions();

	emit InformationSet(serverurl, serverport, proxyurl, proxyport, filenameformat, newdir, copyacrossfilesystems, username, password);
}

void QMBConfig::SaveDefaults() {
	//call Applied to make sure the currently selected options (that the widgets show)
	//are actually in the variables
	this->Applied();

	this->settings.setPath("qmbtagger.sf.net", "QMBTagger");

	//prepend QMBTagger to all keys, results in $HOME/.qt/qmbtaggerrc file
	this->settings.beginGroup( "/QMBTagger");

		this->settings.writeEntry( "/server/url", this->serverurl);
		this->settings.writeEntry( "/server/port", this->serverport);
		this->settings.writeEntry( "/proxy/url", this->proxyurl);
		this->settings.writeEntry( "/proxy/port", this->proxyport);
		this->settings.writeEntry( "/output/format", this->filenameformat);
		this->settings.writeEntry( "/output/directory", this->newdir);
		this->settings.writeEntry( "/output/crossfs", this->copyacrossfilesystems);
		this->settings.writeEntry( "/id/username", this->username);
		this->settings.writeEntry( "/id/password", this->password);

		//emit the signal to save TRMS
		emit SaveTRMs();

	//end QMBTagger group
	this->settings.endGroup();
}

void QMBConfig::Applied()
{
	//just hide the window, save the settings, then emit an InformationSet signal
	this->hide();
	//use latin1() to assure we get a real copy instead of a reference
	this->serverurl = this->textserverurl->text().latin1();
	this->serverport = this->spinserverport->value();
	this->proxyurl = this->textproxyurl->text().latin1();
	this->proxyport = this->spinproxyport->value();
	this->filenameformat = this->textfilenameformat->text().latin1();
	this->newdir = this->textnewdir->text().latin1();
	if (!this->ValidateDir(this->newdir)) {
		this->newdir = "";
		this->textnewdir->setText(this->newdir);
	}
	this->copyacrossfilesystems = this->checkcrossfilesystem->isChecked();
	this->username = this->textusername->text().latin1();
	this->password = this->textpassword->text().latin1();

	emit InformationSet(serverurl, serverport, proxyurl, proxyport, filenameformat, newdir, copyacrossfilesystems, username, password);
}

void QMBConfig::Cancelled()
{
	//hide the window
	this->hide();
	//copy the saved values back
	this->RestoreOptions();
}

void QMBConfig::SelectOutputDir() {
	QString savedir = QFileDialog::getExistingDirectory(QDir::homeDirPath(), this, "getsavedir", "Select save directory...", TRUE);

	textnewdir->setText(savedir);
}

bool QMBConfig::ValidateDir(QString dirpath) {
	QDir dir(dirpath);

	return dir.exists();
}

void QMBConfig::RestoreOptions() {
	//copy the saved values back
	this->textserverurl->setText(serverurl);
	this->spinserverport->setValue(serverport);
	this->textproxyurl->setText(proxyurl);
	this->spinproxyport->setValue(proxyport);
	this->textfilenameformat->setText(filenameformat);
	this->textnewdir->setText(newdir);
	this->checkcrossfilesystem->setChecked(copyacrossfilesystems);
	this->textusername->setText(username);
	this->textpassword->setText(password);
}
