/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#ifndef _QMBCONFIG_H_
#define _QMBCONFIG_H_

#include <qvbox.h>
#include <qsettings.h>
class QString;
class QLineEdit;
class QSpinBox;
class QCheckBox;
	
class QMBConfig : public QVBox {
	Q_OBJECT
public:
	QMBConfig(QWidget* parent = 0, const char* name = 0, WFlags f = 0);
	~QMBConfig();

	QSettings* GetSettings();
	void EmitInformation();

private slots:
	void LoadDefaults();
	void SaveDefaults();
	void Applied();
	void Cancelled();
	void SelectOutputDir();

signals:
	void InformationSet(QString /*serverurl*/, int /*serverport*/, QString /*proxyurl*/, int /*proxyport*/, QString /*filenameformat*/, QString /*newdir*/, bool /*copyacrossfilesystems*/, QString /*username*/, QString /*password*/);
	void LoadTRMs();
	void SaveTRMs();

private:
	bool ValidateDir(QString dirpath);
	void RestoreOptions();

	QSettings settings;

	//saved vals
	QString serverurl;
	int serverport;
	QString proxyurl;
	int proxyport;
	QString filenameformat;
	QString newdir;
	bool copyacrossfilesystems;
	QString username;
	QString password;

	//the few widgets we need to save pointers to so we can reset vals on a cancel
	QLineEdit* textserverurl;
	QSpinBox* spinserverport;
	QLineEdit* textproxyurl;
	QSpinBox* spinproxyport;
	QLineEdit* textfilenameformat;
	QLineEdit* textnewdir;
	QCheckBox* checkcrossfilesystem;
	QLineEdit* textusername;
	QLineEdit* textpassword;
};

#endif //define _QMBCONFIG_H_
