/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#ifndef _QMBFILE_H_
#define _QMBFILE_H_

class QString;
class MusicBrainz;

#include "qmbmetadata.h"
#include <musicbrainz/mb_c.h>

//this is for specific types of files that need to use std::ios, gcc < 3 uses a different namespace
#if __GNUC__ == 3 || __GNUC__ > 3
#define STD_IOS_BASE_IN std::ios_base::in
#define STD_IOS_BASE_OUT std::ios_base::out
#define STD_IOS_BASE_BINARY std::ios_base::binary
#define STD_IOS_BASE_TRUNC std::ios_base::trunc
#else
#define STD_IOS_BASE_IN std::ios::in
#define STD_IOS_BASE_OUT std::ios::out
#define STD_IOS_BASE_BINARY std::ios::binary
#define STD_IOS_BASE_TRUNC std::ios::trunc
#endif

class QMBFile
{
public:
	QMBFile();
	virtual ~QMBFile() = 0;

	virtual int Extract(QString filename) = 0;
	int CreateID(trm_t &trm);
	int QueryServer(MusicBrainz* mb);
	virtual int Output(int resultChoice, QString format, QString newdir, bool crossfs) = 0;
	void CopySelectedMetadata(int resultChoice);
	void StripURL(QString &url);
	bool wasManuallyTagged();
	void setManuallyTagged(bool t);
	void ReplaceFilenameWildcards(QString &filename);
	virtual QString GetFileExtension() = 0;
	bool CreateDirectory(QString filename);

	long bufferlen;
	char* buffer;
	int channels;
	long sampleRate;
	unsigned int bits_per_sample;
	double songlength;
	char signature[17];
	char ascii_sig[37];
	QString fullfilename;
	QString serverurl;
	int serverport;
	bool manTagged;
	
	//metadata
	QMBMetadata metadata;
	QMBMetadata* queryresults;
	int numresults;
};

#endif //define _QMBFILE_H_
