/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#ifndef _QMBFILEVIEW_H_
#define _QMBFILEVIEW_H_

#include <vector>
#include <qsplitter.h>
#include "qmbsubmit.h"
#include <musicbrainz/mb_c.h>

class QPopupMenu;
class QSettings;
class QStringList;
class QString;
class QListBox;
class QListBoxItem;
class QListView;
class QListViewItem;
class QMBFile;
class QMBTagBox;
class QMBFileViewFileItem;
class MusicBrainz;

using namespace std;

class QMBFileView : public QSplitter
{
	Q_OBJECT
public:
	QMBFileView(QWidget* parent = 0, const char* name = 0);
	~QMBFileView();

	int NumSavedTRMs();
	int FileCount();
	bool Authenticate();
	void TransmitSettings(QSettings* newsettings);

public slots:
	void SetFilenames(const QStringList & filelist);
	bool AddDirectory(QString dirname, bool recursive);
	void QueryServer();
	void OutputSelected();
	void setOptions(QString server, int port, QString proxy, int pport, QString filenameformat, QString outputdir, bool copycrossfs, QString user, QString pass);
	void SubmitTRMs();
	void showContextMenu(QListBoxItem* item, const QPoint & pos);
	void ManuallyTag();
	void ManuallyTag(QListBoxItem* item);
	void AppliedTag();
	void RemoveSong();

signals:
	void ExtractedFileData(const QString&);
	void CreatedId(const QString&);
	void QueriedServer(const QString&);
	void AddedResultsToList(const QString&);
	void AddedResultsToList();
	void OutputtedFile(const QString&);
	void FinishedQuerying(const QString&);
	void FinishedOutputting(const QString&);
	void SubmittedTRMs(const QString&);
	//dummy signals to pass on signal from QMBTagger (which gets it from QMBConfig)
	void SaveTRMs();
	void LoadTRMs();
	
protected:
	virtual void dragEnterEvent(QDragEnterEvent* event);
	virtual void dropEvent(QDropEvent * e);
	
private slots:
	void resultClicked(QListViewItem*);
	void songClicked(QListBoxItem*);

private:
	QString format;
	QString newdir;
	bool crossfs;
	vector<QMBFileViewFileItem*> listparents;
	QStringList filenames;
	MusicBrainz* mb;
	trm_t trm;
	QMBSubmit submit;
	bool authenticated;
	QString username, password;
	QMBTagBox* tagbox;
	QPopupMenu* contextMenu;
	QMBFileViewFileItem* contextItem;
	QSplitter* details;
	QListBox* songs;
	QListView* metalist;
	QListView* results;
};

#endif //define _QMBFILEVIEW_H_
