/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include <config.h>
#include "qmbfileviewfileitem.h"
#include "qmbfileviewresultitem.h"
#include "qmbfile.h"
#include "qmbmetadata.h"

#include "qmbvorbisfile.h"
#include "qmbflacfile.h"
#include "qmbmp3file.h"

#include <qlistview.h>

QMBFileViewFileItem::QMBFileViewFileItem(QListBox * parent, const QString &text)
	: QListBoxText(parent, text), showing(true)
{
}

QMBFileViewFileItem::~QMBFileViewFileItem() {
	if (file != NULL)
		delete file;
}

QMBMetadata QMBFileViewFileItem::GetFileMetadata() {
	return file->metadata;
}

QMBFile* QMBFileViewFileItem::GetFile() {
	return file;
}
	
void QMBFileViewFileItem::SetFile(QMBFile* f) {
	file = f;
}

QMBFile* QMBFileViewFileItem::CreateFile(QString filename) {

#if USE_OGG == 1
		if (filename.right(3) == "ogg") {
			file = new QMBVorbisFile();
		} else 
#endif
#if USE_MP3 == 1
		if (filename.right(3) == "mp3") {
			file = new QMBMP3File();
		} else
#endif
#if USE_FLAC == 1
		if (filename.right(4) == "flac") {
			file = new QMBFLACFile();
		} else
#endif
		{
			qWarning("QMBFileViewFileItem::CreateFile(): The current file (%s) doesn't appear to be supported.", filename.latin1());
			file = NULL;
		}
		
		return file;
}

void QMBFileViewFileItem::AddChild(QMBFileViewResultItem* child) {
	children.push_back(child);
}

QMBFileViewResultItem* QMBFileViewFileItem::GetSelectedChild() {
	for(unsigned int index = 0; index < children.size(); index++) {
		if (children[index]->isSelected() == TRUE)
			return children[index];
	}
	return NULL;
}

void QMBFileViewFileItem::ShowChildren(QListView* listview) {
	if (showing == false && children.size() > 0) {
		for(unsigned int index = 0; index < children.size(); index++) {
			listview->insertItem(children[index]);
		}
		showing = true;
	}
}

void QMBFileViewFileItem::HideChildren(QListView* listview) {
	if (showing == true && children.size() > 0) {
		for(unsigned int index = 0; index < children.size(); index++) {
			listview->takeItem(children[index]);
		}
		showing = false;
	}
}

void QMBFileViewFileItem::ClearChildren(QListView* listview) {
	for (unsigned int index = 0; index < children.size(); index++) {
		listview->takeItem(children[index]);
		delete children[index];
	}
	children.clear();
}

int QMBFileViewFileItem::ChildCount() {
	return children.size();
}

//Sets only the given result item to be selected
void QMBFileViewFileItem::SetSingleSelection(QListView* listview, QListViewItem* selected) {
	for(unsigned int index = 0; index < children.size(); index++) {
		if (children[index] != selected)
			listview->setSelected(children[index], FALSE);
		else
			listview->setSelected(children[index], TRUE);
	}
}
