/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include "qmbsubmit.h"
#include <config.h>
#include <musicbrainz/musicbrainz.h>
#include "iostream"

QMBSubmit::QMBSubmit()
: list(), settings(NULL)
{

}

QMBSubmit::~QMBSubmit() {

}

void QMBSubmit::Add(QString trackID, QString trmID) {
	if (trmID == "00000000-0000-0000-0000-000000000000" ||
		trmID == "40400000-0000-0000-0000-000000000000" ||
		trmID == "ffffffff-ffff-ffff-ffff-ffffffffffff" ||
		trmID.length() != 36 || trackID.length() != 36) {


		qWarning("QMBSubmit::Add(): Error adding song to list (%s, %s).", trackID.latin1(), trmID.latin1());
		return;
	}

	list[trackID] = trmID;
}

void QMBSubmit::Remove(QString trackID) {

	QMap<QString, QString>::Iterator it;

	for(it = list.begin(); it != list.end(); ++it) {
		if (it.key() == trackID) {
			list.erase(it);
			return;
		}
	}

}

void QMBSubmit::Clear() {
	list.clear();
}

int QMBSubmit::Count() {
	return list.count();
}

void QMBSubmit::SetSettings(QSettings* newsettings) {
	this->settings = newsettings;
}

bool QMBSubmit::Submit(MusicBrainz* mb) {
	if (this->Count() < 1) {
		qWarning("QMBSubmit::Submit(): No TRMs are currently stored for submission.");
		return TRUE;
	}

	QMap<QString, QString>::Iterator it;

	QString rdf = "<mq:SubmitTRMList>\n";
	rdf.append("<mm:trmidList>\n");
	rdf.append("<rdf:Bag>\n");

	for (it = list.begin(); it != list.end(); ++it) {
		rdf.append("<rdf:li>\n");
		rdf.append("<mq:trmTrackPair>\n");
		rdf.append("<mm:trmid>");

		rdf.append(it.data());
		rdf.append("</mm:trmid>\n<mm:trackid>");
		rdf.append(it.key());

		rdf.append("</mm:trackid>\n</mq:trmTrackPair>\n</rdf:li>\n");

	}

	rdf.append("</rdf:Bag>\n");
	rdf.append("</mm:trmidList>\n");
	rdf.append("<mq:sessionId>@SESSID@</mq:sessionId>\n");
	rdf.append("<mq:sessionKey>@SESSKEY@</mq:sessionKey>\n<mq:clientVersion>");

	rdf.append(PACKAGE);
	rdf.append("/");
	rdf.append(VERSION);

	rdf.append("</mq:clientVersion>\n");
	rdf.append("</mq:SubmitTRMList>\n");

	if (!mb->Query((const std::string) rdf.latin1())) {
		std::string error;
		mb->GetQueryError(error);
		qWarning("QMBSubmit::Submit(): Submission query failed: %s", error.c_str());
		return FALSE;
	}

	this->Clear();

	return TRUE;
}

void QMBSubmit::Save() {
	this->settings->beginGroup( "/TRM");

		this->settings->writeEntry( "/count", this->Count());

		QMap<QString, QString>::Iterator it;
		int curitem = 0;

		for(it = list.begin(); it != list.end(); ++it) {
			this->settings->writeEntry( QString("/key") + QString::number(curitem), it.key());
			this->settings->writeEntry( QString("/data") + QString::number(curitem), it.data());
			curitem++;
		}

	this->settings->endGroup();
}

void QMBSubmit::Load() {
	this->settings->beginGroup( "/TRM");

		int count = this->settings->readNumEntry( "/count", 0);

		if (!this->settings->removeEntry("/count")) qWarning("QMBSubmit::Load(): /count not deleted");

		for (int curitem = 0; curitem < count; curitem++) {
			QString key = this->settings->readEntry( QString("/key") + QString::number(curitem));
			this->settings->removeEntry( QString("/key") + QString::number(curitem));
			QString data = this->settings->readEntry( QString("/data") + QString::number(curitem));
			this->settings->removeEntry( QString("/data") + QString::number(curitem));
			this->Add(key, data);
		}

	this->settings->endGroup();
}
