/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include "qmbtagbox.h"
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qframe.h>
#include <qlayout.h>

QMBTagBox::QMBTagBox(QWidget* parent, const char* name, WFlags f)
	:QVBox(parent, name, f)
{
	QFrame* gridframe = new QFrame(this);
	QGridLayout* gridlayout = new QGridLayout(gridframe, 10, 4);
	
	QLabel* labelArtistName = new QLabel("Artist Name:", gridframe);
	gridlayout->addWidget(labelArtistName, 0, 0);
	ArtistName = new QLineEdit(gridframe);
	gridlayout->addWidget(ArtistName, 0, 2);
	gridlayout->addRowSpacing(1, 10);
	QLabel* labelAlbumName = new QLabel("Album Name:", gridframe);
	gridlayout->addWidget(labelAlbumName, 2, 0);
	AlbumName = new QLineEdit(gridframe);
	gridlayout->addWidget(AlbumName, 2, 2);
	gridlayout->addRowSpacing(3, 10);
	QLabel* labelTrackName = new QLabel("Track Name:", gridframe);
	gridlayout->addWidget(labelTrackName, 4, 0);
	TrackName = new QLineEdit(gridframe);
	gridlayout->addWidget(TrackName, 4, 2);
	gridlayout->addRowSpacing(5, 10);
	QLabel* labelTrackNum = new QLabel("Track Number:", gridframe);
	gridlayout->addWidget(labelTrackNum, 6, 0);
	TrackNum = new QLineEdit(gridframe);
	gridlayout->addWidget(TrackNum, 6, 2);
	gridlayout->addRowSpacing(7, 10);
	
	QPushButton* apply = new QPushButton("Apply", gridframe);
	gridlayout->addWidget(apply, 8, 0);
	QPushButton* cancel = new QPushButton("Cancel", gridframe);
	gridlayout->addWidget(cancel, 8, 2);
	
	gridlayout->addColSpacing(1, 5);
	gridlayout->setRowStretch(9, 1);
	gridlayout->addColSpacing(2, 250);
	gridlayout->setColStretch(3, 1);
	
	connect(apply, SIGNAL(clicked()), this, SLOT(Applied()));
	connect(cancel, SIGNAL(clicked()), this, SLOT(Cancelled()));
}

QMBTagBox::~QMBTagBox() {
	
}

void QMBTagBox::SetMetadataPtr(QMBMetadata* md) {
	metadata = md;
	ArtistName->setText(metadata->GetArtistName());
	AlbumName->setText(metadata->GetAlbumName());
	TrackName->setText(metadata->GetTrackName());
	TrackNum->setText(metadata->GetTrackNumber());
}
	
void QMBTagBox::Applied() {
	if (metadata != NULL) {
		metadata->SetArtistName(ArtistName->text());
		metadata->SetAlbumName(AlbumName->text());
		metadata->SetTrackName(TrackName->text());
		metadata->SetTrackNumber(TrackNum->text());
	}
	
	emit ChangedMetadata();
	
	this->hide();
}

void QMBTagBox::Cancelled() {
	this->hide();
}
