/*-----------------------------------------------------------------------
QMBTagger - Qt Based MusicBrainz MP3/OGG/FLAC Tagger
Copyright (C) 2003,2004 Ewen Cheslack-Postava

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-----------------------------------------------------------------------*/

#include <config.h>

#include "qmbtagger.h"
#include "qmbfileview.h"
#include "qmbconfig.h"
#include "qcdlookup.h"
#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qaction.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qdir.h>
#include <qsettings.h>

#include "icons/open.xpm"
#include "icons/open_dir.xpm"
#include "icons/save.xpm"
#include "icons/query.xpm"
#include "icons/config.xpm"
#include "icons/submit.xpm"
#include "icons/cd.xpm"

QMBTagger::QMBTagger(int argc, char** argv, QWidget* parent, const char* name, WFlags f)
	:QMainWindow(parent, name, f), curdir("")
{
	//set the caption for this window
	this->setCaption("QMBTagger");

	//central widget (our song list)
	fileview = new QMBFileView(this);
	this->setCentralWidget(fileview);

	//configuration window
	config = new QMBConfig();
	config->setCaption("QMBTagger Configuration");
	
	//cdlookup window
	cdlookup = new QCDLookup();
	cdlookup->setCaption("CD Lookup");
	
	//actions
	QAction* fileOpenFiles = new QAction("Open Files", QPixmap((const char** )open_xpm), "&Open...", CTRL+Key_O, this, "open");
	connect(fileOpenFiles, SIGNAL(activated()), this, SLOT(openfiles()));

	QAction* fileOpenDirectory = new QAction("Open Directory", QPixmap((const char** )open_dir_xpm), "Open &Directory...", CTRL+Key_D, this, "opendir");
	connect(fileOpenDirectory, SIGNAL(activated()), this, SLOT(opendirectory()));

	QAction* fileQueryFiles = new QAction("Query Files", QPixmap((const char** )query_xpm), "&Query...", CTRL+Key_Q, this, "query");
	connect(fileQueryFiles, SIGNAL(activated()), this->fileview, SLOT(QueryServer()));

	QAction* fileSaveFiles = new QAction("Save Files", QPixmap((const char** )save_xpm), "&Save...", CTRL+Key_S, this, "save");
	connect(fileSaveFiles, SIGNAL(activated()), this->fileview, SLOT(OutputSelected()));

	QAction* fileSubmitTRMs = new QAction("Submit TRMs", QPixmap((const char** )submit_xpm), "S&ubmit...", CTRL+Key_U, this, "submit");
	connect(fileSubmitTRMs, SIGNAL(activated()), this->fileview, SLOT(SubmitTRMs()));

	QAction* fileExit = new QAction("Exit", "E&xit...", CTRL+Key_X, this, "exit");
	connect(fileExit, SIGNAL(activated()), qApp, SLOT(closeAllWindows()));

	QAction* toolsCDLookup = new QAction("CD LookUp", QPixmap((const char**) cd_xpm), "&CD LookUp", CTRL+Key_C, this, "cdlookup");
	connect(toolsCDLookup, SIGNAL(activated()), cdlookup, SLOT(show()));
	connect(toolsCDLookup, SIGNAL(activated()), cdlookup, SLOT(CDLookup()));

	QAction* toolsOptions = new QAction("Options", QPixmap((const char** )config_xpm), "O&ptions", CTRL+Key_P, this, "options");
	connect(toolsOptions, SIGNAL(activated()), config, SLOT(show()));

	QAction* helpAbout = new QAction("About", "&About...", CTRL+Key_H, this, "about");
	connect(helpAbout, SIGNAL(activated()), this, SLOT(about()));

	QAction* helpAboutQt = new QAction("About Qt", "About &Qt...", CTRL+Key_Q, this, "aboutqt");
	connect(helpAboutQt, SIGNAL(activated()), this, SLOT(aboutqt()));

	//popup menus (standard menus)
	QPopupMenu* filemenu = new QPopupMenu(this);
	this->menuBar()->insertItem("&File", filemenu);
	fileOpenFiles->addTo(filemenu);
	fileOpenDirectory->addTo(filemenu);
	filemenu->insertSeparator();
	fileQueryFiles->addTo(filemenu);
	filemenu->insertSeparator();
	fileSaveFiles->addTo(filemenu);
	fileSubmitTRMs->addTo(filemenu);
	filemenu->insertSeparator();
	fileExit->addTo(filemenu);

	QPopupMenu* toolsmenu = new QPopupMenu(this);
	this->menuBar()->insertItem("&Tools", toolsmenu);
	toolsCDLookup->addTo(toolsmenu);
	toolsmenu->insertSeparator();
	toolsOptions->addTo(toolsmenu);

	QPopupMenu* helpmenu = new QPopupMenu(this);
	this->menuBar()->insertItem("&Help", helpmenu);
	helpAbout->addTo(helpmenu);
	helpAboutQt->addTo(helpmenu);

	//toolbar
	QToolBar* ops = new QToolBar(this, "ops");
	fileOpenFiles->addTo(ops);
	fileOpenDirectory->addTo(ops);
	ops->addSeparator();
	fileQueryFiles->addTo(ops);
	ops->addSeparator();
	fileSaveFiles->addTo(ops);
	fileSubmitTRMs->addTo(ops);
	ops->addSeparator();
	toolsCDLookup->addTo(ops);
	toolsOptions->addTo(ops);

	//statusbar
	connect( fileview, SIGNAL(ExtractedFileData(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(CreatedId(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(QueriedServer(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(AddedResultsToList(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(OutputtedFile(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(FinishedQuerying(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(FinishedQuerying(const QString&)), this->statusBar(), SLOT(message(const QString&)) );
	connect( fileview, SIGNAL(SubmittedTRMs(const QString&)), this->statusBar(), SLOT(message(const QString&)) );

	//connect the config to the fileview
	connect( config, SIGNAL(InformationSet(QString, int, QString, int, QString, QString, bool, QString, QString)), fileview, SLOT(setOptions(QString, int, QString, int, QString, QString, bool, QString, QString)));

	//artificially cause config.InformationSet(...) to be called so that we can get our initial settings put in here
	config->EmitInformation();
	
	//transmit the QSettings* so that the QMBSubmit object can use it
	this->transmitSettings();

	//for LoadTRMs() we take an extra step with the dummy signal in qmbtagger so that we can emit the LoadTRMs() signal which would otherwise
	//be lost during the QMBConfig::QMBConfig() because no connection has been made until now (which allows us to still load the TRMs on startup
	connect( config, SIGNAL(LoadTRMs()), this, SIGNAL(LoadTRMs()));
	connect( this, SIGNAL(LoadTRMs()), fileview, SIGNAL(LoadTRMs()));
	//now call a function to setup for, emit, and clean up after, the LoadTRMs signal
	this->LoadOldInfo();

	//for SaveTRMs() we take an extra step with the dummy signal in qmbtagger so that we can also save them without saving options
	connect( config, SIGNAL(SaveTRMs()), this, SIGNAL(SaveTRMs()));
	connect( this, SIGNAL(SaveTRMs()), fileview, SIGNAL(SaveTRMs()));
	
	//deal with command line parameters
	ParseCommandLine(argc, argv);
}

QMBTagger::~QMBTagger()
{
	delete fileview;
	delete config;
	delete cdlookup;
}

void QMBTagger::LoadOldInfo() {
	QSettings* settings = this->config->GetSettings();

	settings->setPath("qmbtagger.sf.net", "QMBTagger");

	//prepend QMBTagger to all keys, results in $HOME/.qt/qmbtaggerrc file
	settings->beginGroup( "/QMBTagger");

		emit LoadTRMs();
		
		this->curdir = settings->readEntry( "/curdir" , QDir::homeDirPath());
	
	//end QMBTagger group
	settings->endGroup();
}

void QMBTagger::openfiles()
{
	QStringList files = QFileDialog::getOpenFileNames("Music Files(" + GetFileTypeString() + ")", curdir, this, "open files dialog", "Select files to tag...");

	fileview->SetFilenames(files);
	
	if (files.size() > 0) {
		QString dirname = files[0];
		curdir = dirname.left(dirname.findRev('/') + 1);
	}
}

void QMBTagger::opendirectory()
{
	QFileDialog* dirdialog = new QFileDialog(curdir, "Music Files(" + GetFileTypeString() + ")", this);
	dirdialog->setMode(QFileDialog::Directory);
	
	if (dirdialog->exec() == QDialog::Rejected) return;
		
	QString dirname = dirdialog->dirPath();

	QDir dir(dirname);

	if (dir.exists() == FALSE) {
		qWarning("Selected Directory (%s) doesn't exist.", dirname.latin1());
		return;
	}

	curdir = dirname;
	
	//check if there are any subdirectories and ask if they want a recursive lookup
	bool recursive = false;
	dir.setFilter(QDir::Dirs);
	QStringList subdirs = dir.entryList();
	if (subdirs.count() > 2) { //we check for more than 2 because . and .. are included in this list
		int result = QMessageBox::warning(this, "Recursive load?", "The selected directory has subdirectories.  Should we look recursively through subdirectories for music files?", "Search Recursively", "Just Load Files From This Directory", QString::null, 0, 1);
		if (result == 0) recursive = true;
	}
	
	bool loadedFiles = fileview->AddDirectory(dirname, recursive);
	if (loadedFiles == false)
		QMessageBox::information( this, "QMBTagger - File Selection", "No files were selected or the directory selected contained no supported files.");
}

void QMBTagger::about()
{
	QMessageBox::about(this, "About " PACKAGE, "<b> About " PACKAGE "</b><br><br>" PACKAGE " " VERSION "<br><br>QMBTagger is a Qt based front end to<br>the MusicBrainz Tagger Client Library.<br><br>http://qmbtagger.sourceforge.net");
}

void QMBTagger::aboutqt()
{
	QMessageBox::aboutQt(this, "About Qt");
}

void QMBTagger::transmitSettings() {
	this->fileview->TransmitSettings(this->config->GetSettings());
}

void QMBTagger::closeEvent(QCloseEvent* e) {
	
	//ask if the user really wants to exit 
	if (fileview->FileCount() > 0) {
		int result = QMessageBox::warning(this, "Exit?", "Are you sure you want to exit?", "Exit", "Keep On Tagging", QString::null, 0, 1);
		if (result == 1) {
			e->ignore();
			return;
		}
	}
	
	QSettings* settings = this->config->GetSettings();
	settings->setPath("qmbtagger.sf.net", "QMBTagger");
	//prepend QMBTagger to all keys, results in $HOME/.qt/qmbtaggerrc file
	settings->beginGroup( "/QMBTagger");
	
	//if they say yes, check if there are still TRMs to be submitted 
	if (fileview->NumSavedTRMs() > 0) {
		int response = QMessageBox::critical(this, "TRM Submission", "There are some unsubmitted TRMs remaining.  Submit now?", "Submit", "Don't Submit", "Save TRMs");
		if (response == 0) //submit TRMs
			fileview->SubmitTRMs();
		else if (response == 2) {//save TRMs
			emit SaveTRMs();
		}
	}

	//save the last directory they used
	if (!curdir.isNull() && !curdir.isEmpty()) {
		settings->writeEntry( "/curdir", this->curdir);
	}

	//end QMBTagger group
	settings->endGroup();

	e->accept();
}

QString QMBTagger::GetFileTypeString() {
	QString types = "";
#if USE_OGG == 1
	types += "*.ogg ";
#endif
#if USE_MP3 == 1
	types += "*.mp3 ";
#endif
#if USE_FLAC == 1
	types += "*.flac ";
#endif
	types.truncate(types.length() - 1);
	
	return types;
}

void QMBTagger::ParseCommandLine(int argc, char** argv) {
	for (int index = 0; index < argc; index++) {
		if (QString(argv[index]) == "-d" && argc > index+1) {
			curdir = argv[index+1];
		}
		else {
			qApp->exit(0);
		}
	}
}
